@@ -4,6 +4,7 @@
 #include "defs.h"
 
 #include <string>
+#include <unordered_map>
 
 struct Flags {
 	int width = 1280, height = 720;
@@ -20,6 +21,7 @@ struct Flags {
 	double cue_out_point_padding_seconds = 0.0;  // Can be changed in the menus.
 	bool cue_out_point_padding_set = false;
 	std::string midi_mapping_filename;  // Empty for none.
+	std::unordered_map<unsigned, std::string> source_labels;
 };
 extern Flags global_flags;
 
