@@ -82,9 +82,16 @@ DeckLinkOutput::DeckLinkOutput(ResourcePool *resource_pool, QSurface *surface, u
 	});
 }
 
+DeckLinkOutput::~DeckLinkOutput()
+{
+	if (output != nullptr) {
+		output->Release();
+	}
+}
+
 bool DeckLinkOutput::set_device(IDeckLink *decklink, IDeckLinkInput *input_arg)
 {
-	input_arg = input;
+	input = input_arg;
 	if (decklink->QueryInterface(IID_IDeckLinkOutput, (void**)&output) != S_OK) {
 		fprintf(stderr, "Warning: Card %u has no outputs\n", card_index);
 		return false;
@@ -124,10 +131,11 @@ bool DeckLinkOutput::set_device(IDeckLink *decklink, IDeckLinkInput *input_arg)
 	return true;
 }
 
-void DeckLinkOutput::start_output(uint32_t mode, int64_t base_pts)
+void DeckLinkOutput::start_output(uint32_t mode, int64_t base_pts, bool is_master_card_arg)
 {
 	assert(output);
 	assert(!playback_initiated);
+	this->is_master_card = is_master_card_arg;
 
 	if (video_modes.empty()) {
 		fprintf(stderr, "ERROR: No matching output modes for %dx%d found\n", width, height);
@@ -217,9 +225,13 @@ void DeckLinkOutput::start_output(uint32_t mode, int64_t base_pts)
 		fprintf(stderr, "Couldn't enable audio output\n");
 		abort();
 	}
-	if (output->BeginAudioPreroll() != S_OK) {
-		fprintf(stderr, "Couldn't begin audio preroll\n");
-		abort();
+	if (is_master_card) {
+		if (output->BeginAudioPreroll() != S_OK) {
+			fprintf(stderr, "Couldn't begin audio preroll\n");
+			abort();
+		}
+	} else {
+		playback_started = true;
 	}
 
 	present_thread = thread([this]{
@@ -246,7 +258,9 @@ void DeckLinkOutput::end_output()
 	present_thread.join();
 	playback_initiated = false;
 
-	output->StopScheduledPlayback(0, nullptr, 0);
+	if (is_master_card) {
+		output->StopScheduledPlayback(0, nullptr, 0);
+	}
 	output->DisableVideoOutput();
 	output->DisableAudioOutput();
 
@@ -258,15 +272,6 @@ void DeckLinkOutput::end_output()
 			frame_freelist.pop();
 		}
 	}
-
-	if (input != nullptr) {
-		input->Release();
-		input = nullptr;
-	}
-	if (output != nullptr) {
-		output->Release();
-		output = nullptr;
-	}
 }
 
 void DeckLinkOutput::send_frame(GLuint y_tex, GLuint cbcr_tex, YCbCrLumaCoefficients output_ycbcr_coefficients, const vector<RefCountedFrame> &input_frames, int64_t pts, int64_t duration)
@@ -350,13 +355,20 @@ void DeckLinkOutput::send_audio(int64_t pts, const std::vector<float> &samples)
 	}
 
 	uint32_t frames_written;
-	HRESULT result = output->ScheduleAudioSamples(int_samples.get(), samples.size() / 2,
-		pts, TIMEBASE, &frames_written);
-	if (result != S_OK) {
-		fprintf(stderr, "ScheduleAudioSamples(pts=%" PRId64 ") failed (result=0x%08x)\n", pts, result);
+	HRESULT result;
+	if (is_master_card) {
+		result = output->ScheduleAudioSamples(int_samples.get(), samples.size() / 2,
+			pts, TIMEBASE, &frames_written);
 	} else {
-		if (frames_written != samples.size() / 2) {
-			fprintf(stderr, "ScheduleAudioSamples() returned short write (%u/%zu)\n", frames_written, samples.size() / 2);
+		result = output->WriteAudioSamplesSync(int_samples.get(), samples.size() / 2,
+			&frames_written);
+	}
+	if (result != S_OK) {
+		fprintf(stderr, "write audio to DeckLink (pts=%" PRId64 ") failed (result=0x%08x)\n", pts, result);
+	} else {
+		// Non-master card is not really synchronized on audio at all, so we don't warn on it.
+		if (frames_written != samples.size() / 2 && is_master_card) {
+			fprintf(stderr, "write audio to DeckLink returned short write (%u/%zu)\n", frames_written, samples.size() / 2);
 		}
 	}
 	metric_decklink_output_scheduled_samples += samples.size() / 2;
@@ -501,8 +513,8 @@ HRESULT DeckLinkOutput::ScheduledFrameCompleted(/* in */ IDeckLinkVideoFrame *co
 	{
 		lock_guard<mutex> lock(frame_queue_mutex);
 		frame_freelist.push(unique_ptr<Frame>(frame));
-		assert(scheduled_frames.front() == frame);
-		scheduled_frames.pop_front();
+		assert(scheduled_frames.count(frame));
+		scheduled_frames.erase(frame);
 		--metric_decklink_output_inflight_frames;
 	}
 
@@ -595,16 +607,24 @@ void DeckLinkOutput::present_thread_func()
 		// Release any input frames we needed to render this frame.
 		frame->input_frames.clear();
 
-		BMDTimeValue pts = frame->pts;
-		BMDTimeValue duration = frame->duration;
-		HRESULT res = output->ScheduleVideoFrame(frame.get(), pts, duration, TIMEBASE);
-		lock_guard<mutex> lock(frame_queue_mutex);
-		if (res == S_OK) {
-			scheduled_frames.push_back(frame.release());  // Owned by the driver now.
-			++metric_decklink_output_inflight_frames;
-		} else {
-			fprintf(stderr, "Could not schedule video frame! (error=0x%08x)\n", res);
+		if (is_master_card) {
+			BMDTimeValue pts = frame->pts;
+			BMDTimeValue duration = frame->duration;
+			HRESULT res = output->ScheduleVideoFrame(frame.get(), pts, duration, TIMEBASE);
+			lock_guard<mutex> lock(frame_queue_mutex);
+			if (res == S_OK) {
+				scheduled_frames.insert(frame.release());  // Owned by the driver now.
+				++metric_decklink_output_inflight_frames;
+			} else {
+				fprintf(stderr, "Could not schedule video frame! (error=0x%08x)\n", res);
 
+				frame_freelist.push(move(frame));
+			}
+		} else {
+			HRESULT res = output->DisplayVideoFrameSync(frame.get());
+			if (res != S_OK) {
+				fprintf(stderr, "Could not schedule video frame! (error=0x%08x)\n", res);
+			}
 			frame_freelist.push(move(frame));
 		}
 	}
