@@ -500,7 +500,7 @@ bool FFmpegCapture::play_video(const string &pathname)
 	} else {
 #ifdef HAVE_SRT
 		// SRT socket, already opened.
-		AVInputFormat *mpegts_fmt = av_find_input_format("mpegts");
+		const AVInputFormat *mpegts_fmt = av_find_input_format("mpegts");
 		format_ctx = avformat_open_input_unique(&FFmpegCapture::read_srt_thunk, this,
 			mpegts_fmt, /*options=*/nullptr,
 			AVIOInterruptCB{ &FFmpegCapture::interrupt_cb_thunk, this });
@@ -530,7 +530,7 @@ bool FFmpegCapture::play_video(const string &pathname)
 
 	// Open video decoder.
 	const AVCodecParameters *video_codecpar = format_ctx->streams[video_stream_index]->codecpar;
-	AVCodec *video_codec = avcodec_find_decoder(video_codecpar->codec_id);
+	const AVCodec *video_codec = avcodec_find_decoder(video_codecpar->codec_id);
 
 	video_timebase = format_ctx->streams[video_stream_index]->time_base;
 	AVCodecContextWithDeleter video_codec_ctx = avcodec_alloc_context3_unique(nullptr);
@@ -574,7 +574,7 @@ bool FFmpegCapture::play_video(const string &pathname)
 			fprintf(stderr, "%s: Cannot fill audio codec parameters\n", pathname.c_str());
 			return false;
 		}
-		AVCodec *audio_codec = avcodec_find_decoder(audio_codecpar->codec_id);
+		const AVCodec *audio_codec = avcodec_find_decoder(audio_codecpar->codec_id);
 		if (audio_codec == nullptr) {
 			fprintf(stderr, "%s: Cannot find audio decoder\n", pathname.c_str());
 			return false;
