@@ -543,6 +543,7 @@ Mixer::Mixer(const QSurfaceFormat &format)
 	if (global_flags.enable_alsa_output) {
 		alsa.reset(new ALSAOutput(OUTPUT_FREQUENCY, /*num_channels=*/2));
 	}
+	output_card_is_master = global_flags.output_card_is_master;
 	if (global_flags.output_card != -1) {
 		desired_output_card_index = global_flags.output_card;
 		set_output_card_internal(global_flags.output_card);
@@ -899,7 +900,7 @@ void Mixer::set_output_card_internal(int card_index)
 		card->jitter_history.clear();
 		card->capture->start_bm_capture();
 		desired_output_video_mode = output_video_mode = card->output->pick_video_mode(desired_output_video_mode);
-		card->output->start_output(desired_output_video_mode, pts_int);
+		card->output->start_output(desired_output_video_mode, pts_int, /*is_master_card=*/output_card_is_master);
 	}
 	output_card_index = card_index;
 	output_jitter_history.clear();
@@ -1286,7 +1287,7 @@ void Mixer::thread_func()
 			DeckLinkOutput *output = cards[output_card_index].output.get();
 			output->end_output();
 			desired_output_video_mode = output_video_mode = output->pick_video_mode(desired_output_video_mode);
-			output->start_output(desired_output_video_mode, pts_int);
+			output->start_output(desired_output_video_mode, pts_int, /*is_master_card=*/output_card_is_master);
 		}
 
 		{
@@ -1299,7 +1300,7 @@ void Mixer::thread_func()
 
 		bool master_card_is_output;
 		unsigned master_card_index;
-		if (output_card_index != -1) {
+		if (output_card_index != -1 && output_card_is_master) {
 			master_card_is_output = true;
 			master_card_index = output_card_index;
 		} else {
@@ -1406,7 +1407,7 @@ void Mixer::thread_func()
 
 bool Mixer::input_card_is_master_clock(unsigned card_index, unsigned master_card_index) const
 {
-	if (output_card_index != -1) {
+	if (output_card_index != -1 && output_card_is_master) {
 		// The output card (ie., cards[output_card_index].output) is the master clock,
 		// so no input card (ie., cards[card_index].capture) is.
 		return false;
