@@ -1435,8 +1435,8 @@ void QuickSyncEncoderImpl::release_gl_resources()
 	has_released_gl_resources = true;
 }
 
-QuickSyncEncoderImpl::QuickSyncEncoderImpl(const std::string &filename, ResourcePool *resource_pool, QSurface *surface, const string &va_display, int width, int height, AVOutputFormat *oformat, X264Encoder *x264_encoder, DiskSpaceEstimator *disk_space_estimator)
-	: current_storage_frame(0), resource_pool(resource_pool), surface(surface), x264_encoder(x264_encoder), frame_width(width), frame_height(height), disk_space_estimator(disk_space_estimator)
+QuickSyncEncoderImpl::QuickSyncEncoderImpl(const std::string &filename, ResourcePool *resource_pool, QSurface *surface, const string &va_display, int width, int height, const AVOutputFormat *oformat, X264Encoder *http_encoder, X264Encoder *disk_encoder, DiskSpaceEstimator *disk_space_estimator)
+	: current_storage_frame(0), resource_pool(resource_pool), surface(surface), x264_http_encoder(http_encoder), x264_disk_encoder(disk_encoder), frame_width(width), frame_height(height), disk_space_estimator(disk_space_estimator)
 {
 	file_audio_encoder.reset(new AudioEncoder(AUDIO_OUTPUT_CODEC_NAME, DEFAULT_AUDIO_OUTPUT_BIT_RATE, oformat));
 	open_output_file(filename);
@@ -1448,9 +1448,11 @@ QuickSyncEncoderImpl::QuickSyncEncoderImpl(const std::string &filename, Resource
 	//print_input();
 
 	if (global_flags.x264_video_to_http || global_flags.x264_video_to_disk) {
-		assert(x264_encoder != nullptr);
+		assert(x264_http_encoder != nullptr);
+		assert(x264_disk_encoder != nullptr);
 	} else {
-		assert(x264_encoder == nullptr);
+		assert(x264_http_encoder == nullptr);
+		assert(x264_disk_encoder == nullptr);
 	}
 
 	enable_zerocopy_if_possible();
@@ -1731,7 +1733,7 @@ void QuickSyncEncoderImpl::open_output_file(const std::string &filename)
 
 	string video_extradata;  // FIXME: See other comment about global headers.
 	if (global_flags.x264_video_to_disk) {
-		video_extradata = x264_encoder->get_global_headers();
+		video_extradata = x264_disk_encoder->get_global_headers();
 	}
 
 	current_file_mux_metrics.reset();
@@ -1747,7 +1749,7 @@ void QuickSyncEncoderImpl::open_output_file(const std::string &filename)
 	metric_current_file_start_time_seconds = get_timestamp_for_metrics();
 
 	if (global_flags.x264_video_to_disk) {
-		x264_encoder->add_mux(file_mux.get());
+		x264_disk_encoder->add_mux(file_mux.get());
 	}
 }
 
@@ -1910,7 +1912,10 @@ void QuickSyncEncoderImpl::pass_frame(QuickSyncEncoderImpl::PendingFrame frame,
 	if (global_flags.uncompressed_video_to_http) {
 		add_packet_for_uncompressed_frame(pts, duration, data);
 	} else if (global_flags.x264_video_to_http || global_flags.x264_video_to_disk) {
-		x264_encoder->add_frame(pts, duration, frame.ycbcr_coefficients, data, received_ts);
+		x264_http_encoder->add_frame(pts, duration, frame.ycbcr_coefficients, data, received_ts);
+	}
+	if (global_flags.x264_separate_disk_encode) {
+		x264_disk_encoder->add_frame(pts, duration, frame.ycbcr_coefficients, data, received_ts);
 	}
 
 	if (v4l_output != nullptr) {
@@ -2012,8 +2017,8 @@ void QuickSyncEncoderImpl::encode_frame(QuickSyncEncoderImpl::PendingFrame frame
 }
 
 // Proxy object.
-QuickSyncEncoder::QuickSyncEncoder(const std::string &filename, ResourcePool *resource_pool, QSurface *surface, const string &va_display, int width, int height, AVOutputFormat *oformat, X264Encoder *x264_encoder, DiskSpaceEstimator *disk_space_estimator)
-	: impl(new QuickSyncEncoderImpl(filename, resource_pool, surface, va_display, width, height, oformat, x264_encoder, disk_space_estimator)) {}
+QuickSyncEncoder::QuickSyncEncoder(const std::string &filename, ResourcePool *resource_pool, QSurface *surface, const string &va_display, int width, int height, const AVOutputFormat *oformat, X264Encoder *http_encoder, X264Encoder *disk_encoder, DiskSpaceEstimator *disk_space_estimator)
+	: impl(new QuickSyncEncoderImpl(filename, resource_pool, surface, va_display, width, height, oformat, http_encoder, disk_encoder, disk_space_estimator)) {}
 
 // Must be defined here because unique_ptr<> destructor needs to know the impl.
 QuickSyncEncoder::~QuickSyncEncoder() {}
