@@ -43,7 +43,7 @@ class X264Encoder;
 
 class QuickSyncEncoderImpl {
 public:
-	QuickSyncEncoderImpl(const std::string &filename, movit::ResourcePool *resource_pool, QSurface *surface, const std::string &va_display, int width, int height, AVOutputFormat *oformat, X264Encoder *x264_encoder, DiskSpaceEstimator *disk_space_estimator);
+	QuickSyncEncoderImpl(const std::string &filename, movit::ResourcePool *resource_pool, QSurface *surface, const std::string &va_display, int width, int height, const AVOutputFormat *oformat, X264Encoder *http_encoder, X264Encoder *disk_encoder, DiskSpaceEstimator *disk_space_estimator);
 	~QuickSyncEncoderImpl();
 	void add_audio(int64_t pts, std::vector<float> audio);
 	bool is_zerocopy() const;
@@ -171,7 +171,8 @@ private:
 	std::mutex file_audio_encoder_mutex;
 	std::unique_ptr<AudioEncoder> file_audio_encoder;
 
-	X264Encoder *x264_encoder;  // nullptr if not using x264.
+	X264Encoder *x264_http_encoder;  // nullptr if not using x264.
+	X264Encoder *x264_disk_encoder;
 	std::unique_ptr<V4LOutput> v4l_output;  // nullptr if not using V4L2 output.
 
 	Mux* stream_mux = nullptr;  // To HTTP.
