@@ -18,7 +18,7 @@ extern "C" {
 using namespace movit;
 using namespace std;
 
-bool display(Block *block, lua_State *L, int idx);
+static bool display(Block *block, lua_State *L, int idx);
 
 EffectType current_type(const Block *block)
 {
@@ -176,6 +176,14 @@ bool Scene::is_noncanonical_chain(size_t chain_idx) const
 					return true;
 				}
 			}
+
+			// Auto white balance is always disabled for image inputs.
+			if (block->white_balance_controller_block != nullptr) {
+				const Block *input = block->white_balance_controller_block;
+				if (input->alternatives[input->chosen_alternative(chain_idx)]->effect_type == IMAGE_INPUT) {
+					return true;
+				}
+			}
 		}
 	}
 	return false;
@@ -201,6 +209,11 @@ int Scene::add_input(lua_State* L)
 		if (lua_isnumber(L, 2)) {
 			block->alternatives.emplace_back(new EffectBlueprint(LIVE_INPUT_YCBCR));
 			block->alternatives.emplace_back(new EffectBlueprint(LIVE_INPUT_YCBCR_WITH_DEINTERLACE));
+#ifdef HAVE_SRT
+			if (global_flags.srt_port >= 0) {
+				block->alternatives.emplace_back(new EffectBlueprint(LIVE_INPUT_YCBCR_PLANAR));
+			}
+#endif
 #ifdef HAVE_CEF
 		} else if (luaL_testudata(L, 2, "HTMLInput")) {
 			block->alternatives.emplace_back(new EffectBlueprint(LIVE_INPUT_BGRA));
@@ -227,34 +240,35 @@ int Scene::add_input(lua_State* L)
 	return wrap_lua_existing_object_nonowned<Block>(L, "Block", block);
 }
 
-void Scene::find_inputs_for_block(lua_State *L, Scene *scene, Block *block)
+Block *Scene::find_block_from_arg(lua_State *L, Scene *scene, int idx)
 {
-	if (lua_gettop(L) == 2) {
+	if (luaL_testudata(L, idx, "Block")) {
+		return *(Block **)luaL_checkudata(L, idx, "Block");
+	} else {
+		EffectBlueprint *blueprint = *(EffectBlueprint **)luaL_checkudata(L, idx, "EffectBlueprint");
+
+		// Search through all the blocks to figure out which one contains this effect.
+		for (Block *block : scene->blocks) {
+			if (find(block->alternatives.begin(), block->alternatives.end(), blueprint) != block->alternatives.end()) {
+				return block;
+			}
+		}
+		luaL_error(L, "Input effect in parameter #%d has not been added to this scene", idx - 1);
+		return nullptr;  // Dead code.
+	}
+}
+
+void Scene::find_inputs_for_block(lua_State *L, Scene *scene, Block *block, int first_input_idx)
+{
+	if (lua_gettop(L) == first_input_idx - 1) {
 		// Implicitly the last added effect.
 		assert(!scene->blocks.empty());
 		block->inputs.push_back(scene->blocks.size() - 1);
 		return;
 	}
 
-	for (int idx = 3; idx <= lua_gettop(L); ++idx) {
-		Block *input_block = nullptr;
-		if (luaL_testudata(L, idx, "Block")) {
-			input_block = *(Block **)luaL_checkudata(L, idx, "Block");
-		} else {
-			EffectBlueprint *blueprint = *(EffectBlueprint **)luaL_checkudata(L, idx, "EffectBlueprint");
-
-			// Search through all the blocks to figure out which one contains this effect.
-			for (Block *block : scene->blocks) {
-				if (find(block->alternatives.begin(), block->alternatives.end(), blueprint) != block->alternatives.end()) {
-					input_block = block;
-					break;
-				}
-			}
-			if (input_block == nullptr) {
-				luaL_error(L, "Input effect in parameter #%d has not been added to this scene", idx - 1);
-			}
-		}
-		block->inputs.push_back(input_block->idx);
+	for (int idx = first_input_idx; idx <= lua_gettop(L); ++idx) {
+		block->inputs.push_back(find_block_from_arg(L, scene, idx)->idx);
 	}
 }
 
@@ -318,6 +332,65 @@ int Scene::add_optional_effect(lua_State* L)
 	return wrap_lua_existing_object_nonowned<Block>(L, "Block", block);
 }
 
+const Block *Scene::find_root_input_block(lua_State *L, const Block *block)
+{
+	if (block->is_input) {
+		assert(block->inputs.size() == 0);
+		return block;
+	}
+
+	const Block *ret = nullptr;
+	for (size_t input_idx : block->inputs) {
+		const Block *parent = find_root_input_block(L, blocks[input_idx]);
+		if (parent != nullptr) {
+			if (ret != nullptr) {
+				luaL_error(L, "add_white_balance() was connected to more than one input");
+			}
+			ret = parent;
+		}
+	}
+	return ret;
+}
+
+int Scene::add_white_balance(lua_State* L)
+{
+	assert(lua_gettop(L) >= 1);
+	Scene *scene = (Scene *)luaL_checkudata(L, 1, "Scene");
+
+	Block *block = new Block;
+	block->declaration_point = get_declaration_point(L);
+	block->idx = scene->blocks.size();
+
+	block->alternatives.push_back(new EffectBlueprint(WHITE_BALANCE_EFFECT));
+	block->alternatives.push_back(new EffectBlueprint(IDENTITY_EFFECT));
+
+	block->canonical_alternative = 1;
+
+	if (lua_gettop(L) == 1) {
+		// The last added effect is implicitly both the input and gives the white balance controller.
+		assert(!scene->blocks.empty());
+		block->inputs.push_back(scene->blocks.size() - 1);
+		block->white_balance_controller_block = scene->find_root_input_block(L, block);
+	} else if (lua_gettop(L) == 2) {
+		// The given effect is both the input and the white balance controller.
+		block->inputs.push_back(find_block_from_arg(L, scene, 2)->idx);
+		block->white_balance_controller_block = scene->find_root_input_block(L, block);
+	} else if (lua_gettop(L) == 3) {
+		// We have explicit input and white balance controller.
+		block->inputs.push_back(find_block_from_arg(L, scene, 2)->idx);
+		block->white_balance_controller_block = find_block_from_arg(L, scene, 3);
+	} else {
+		luaL_error(L, "add_white_balance([input], [white_balance_controller]) takes zero, one or two arguments");
+	}
+	if (block->white_balance_controller_block == nullptr || !block->white_balance_controller_block->is_input) {
+		luaL_error(L, "add_white_balance() does not get its white balance from an input");
+	}
+
+	scene->blocks.push_back(block);
+
+	return wrap_lua_existing_object_nonowned<Block>(L, "Block", block);
+}
+
 Effect *Scene::instantiate_effects(const Block *block, size_t chain_idx, Scene::Instantiation *instantiation)
 {
 	// Find the chosen alternative for this block in this instance.
@@ -427,6 +500,24 @@ int Scene::finalize(lua_State* L)
 	return 0;
 }
 
+int find_card_to_connect(Theme *theme, lua_State *L, const Block *block)
+{
+	if (block->signal_type_to_connect == Block::CONNECT_SIGNAL) {
+		return theme->map_signal_to_card(block->signal_to_connect);
+#ifdef HAVE_CEF
+	} else if (block->signal_type_to_connect == Block::CONNECT_CEF) {
+		return block->cef_to_connect->get_card_index();
+#endif
+	} else if (block->signal_type_to_connect == Block::CONNECT_VIDEO) {
+		return block->video_to_connect->get_card_index();
+	} else if (block->signal_type_to_connect == Block::CONNECT_NONE) {
+		luaL_error(L, "An input in a scene was not connected to anything (forgot to call display())");
+	} else {
+		assert(false);
+	}
+	return -1;
+}
+
 std::pair<movit::EffectChain *, std::function<void()>>
 Scene::get_chain(Theme *theme, lua_State *L, unsigned num, const InputState &input_state)
 {
@@ -435,13 +526,54 @@ Scene::get_chain(Theme *theme, lua_State *L, unsigned num, const InputState &inp
 	InputStateInfo info(input_state);
 	for (Block *block : blocks) {
 		if (block->is_input && block->signal_type_to_connect == Block::CONNECT_SIGNAL) {
-			EffectType chosen_type = current_type(block);
-			assert(chosen_type == LIVE_INPUT_YCBCR || chosen_type == LIVE_INPUT_YCBCR_WITH_DEINTERLACE);
-			if (info.last_interlaced[block->signal_to_connect]) {
+			int card_index = theme->map_signal_to_card(block->signal_to_connect);
+			if (info.last_interlaced[card_index]) {
+				assert(info.last_pixel_format[card_index] == bmusb::PixelFormat_8BitYCbCr ||
+				       info.last_pixel_format[card_index] == bmusb::PixelFormat_10BitYCbCr);
 				block->currently_chosen_alternative = find_index_of(block, LIVE_INPUT_YCBCR_WITH_DEINTERLACE);
+			} else if (info.last_pixel_format[card_index] == bmusb::PixelFormat_8BitYCbCrPlanar) {
+				block->currently_chosen_alternative = find_index_of(block, LIVE_INPUT_YCBCR_PLANAR);
+			} else if (info.last_pixel_format[card_index] == bmusb::PixelFormat_8BitBGRA) {
+				block->currently_chosen_alternative = find_index_of(block, LIVE_INPUT_BGRA);
 			} else {
 				block->currently_chosen_alternative = find_index_of(block, LIVE_INPUT_YCBCR);
 			}
+			if (block->currently_chosen_alternative == -1) {
+				fprintf(stderr, "ERROR: Input connected to a video card pixel format that it was not ready for.\n");
+				abort();
+			}
+		}
+	}
+
+	// Find all auto white balance blocks, turn on and off the effect as needed,
+	// and fetch the actual white balance set (it is stored in Theme).
+	map<Block *, array<float, 3>> white_balance;
+	for (size_t block_idx = 0; block_idx < blocks.size(); ++block_idx) {
+		Block *block = blocks[block_idx];
+		const Block *input = block->white_balance_controller_block;
+		if (input == nullptr) {
+			continue;  // Not an auto white balance block.
+		}
+
+		EffectType chosen_type = current_type(input);
+		if (chosen_type == IMAGE_INPUT) {
+			// Image inputs never get white balance applied.
+			block->currently_chosen_alternative = find_index_of(block, IDENTITY_EFFECT);
+			continue;
+		}
+
+		assert(chosen_type == LIVE_INPUT_YCBCR ||
+		       chosen_type == LIVE_INPUT_YCBCR_WITH_DEINTERLACE ||
+		       chosen_type == LIVE_INPUT_YCBCR_PLANAR ||
+		       chosen_type == LIVE_INPUT_BGRA);
+		int card_idx = find_card_to_connect(theme, L, input);
+		RGBTriplet wb = theme->get_white_balance_for_card(card_idx);
+		if (fabs(wb.r - 1.0) < 1e-3 && fabs(wb.g - 1.0) < 1e-3 && fabs(wb.b - 1.0) < 1e-3) {
+			// Neutral white balance.
+			block->currently_chosen_alternative = find_index_of(block, IDENTITY_EFFECT);
+		} else {
+			block->currently_chosen_alternative = find_index_of(block, WHITE_BALANCE_EFFECT);
+			white_balance.emplace(block, array<float, 3>{ wb.r, wb.g, wb.b });
 		}
 	}
 
@@ -471,7 +603,7 @@ Scene::get_chain(Theme *theme, lua_State *L, unsigned num, const InputState &inp
 	const Scene::Instantiation &instantiation = chains[chain_idx];
 	EffectChain *effect_chain = instantiation.chain.get();
 
-	map<LiveInputWrapper *, int> signals_to_connect;
+	map<LiveInputWrapper *, int> cards_to_connect;
 	map<ImageInput *, string> images_to_select;
 	map<pair<Effect *, string>, int> int_to_set;
 	map<pair<Effect *, string>, float> float_to_set;
@@ -480,24 +612,12 @@ Scene::get_chain(Theme *theme, lua_State *L, unsigned num, const InputState &inp
 	for (const auto &index_and_input : instantiation.inputs) {
 		Block *block = blocks[index_and_input.first];
 		EffectType chosen_type = current_type(block);
-		LiveInputWrapper *input = index_and_input.second;
 		if (chosen_type == LIVE_INPUT_YCBCR ||
 		    chosen_type == LIVE_INPUT_YCBCR_WITH_DEINTERLACE ||
 		    chosen_type == LIVE_INPUT_YCBCR_PLANAR ||
 		    chosen_type == LIVE_INPUT_BGRA) {
-			if (block->signal_type_to_connect == Block::CONNECT_SIGNAL) {
-				signals_to_connect.emplace(input, block->signal_to_connect);
-#ifdef HAVE_CEF
-			} else if (block->signal_type_to_connect == Block::CONNECT_CEF) {
-				signals_to_connect.emplace(input, block->cef_to_connect->get_card_index());
-#endif
-			} else if (block->signal_type_to_connect == Block::CONNECT_VIDEO) {
-				signals_to_connect.emplace(input, block->video_to_connect->get_card_index());
-			} else if (block->signal_type_to_connect == Block::CONNECT_NONE) {
-				luaL_error(L, "An input in a scene was not connected to anything (forgot to call display())");
-			} else {
-				assert(false);
-			}
+			LiveInputWrapper *input = index_and_input.second;
+			cards_to_connect.emplace(input, find_card_to_connect(theme, L, block));
 		}
 	}
 	for (const auto &index_and_input : instantiation.image_inputs) {
@@ -530,6 +650,9 @@ Scene::get_chain(Theme *theme, lua_State *L, unsigned num, const InputState &inp
 			for (const auto &key_and_tuple : block->float_parameters) {
 				float_to_set.emplace(make_pair(effect, key_and_tuple.first), key_and_tuple.second);
 			}
+			if (white_balance.count(block)) {
+				vec3_to_set.emplace(make_pair(effect, "neutral_color"), white_balance[block]);
+			}
 			for (const auto &key_and_tuple : block->vec3_parameters) {
 				vec3_to_set.emplace(make_pair(effect, key_and_tuple.first), key_and_tuple.second);
 			}
@@ -573,13 +696,13 @@ Scene::get_chain(Theme *theme, lua_State *L, unsigned num, const InputState &inp
 
 	lua_pop(L, 1);
 
-	auto setup_chain = [L, theme, signals_to_connect, images_to_select, int_to_set, float_to_set, vec3_to_set, vec4_to_set, input_state]{
+	auto setup_chain = [L, theme, cards_to_connect, images_to_select, int_to_set, float_to_set, vec3_to_set, vec4_to_set, input_state]{
 		lock_guard<mutex> lock(theme->m);
 
-		// Set up state, including connecting signals.
-		for (const auto &input_and_signal : signals_to_connect) {
-			LiveInputWrapper *input = input_and_signal.first;
-			input->connect_signal_raw(input_and_signal.second, input_state);
+		// Set up state, including connecting cards.
+		for (const auto &input_and_card : cards_to_connect) {
+			LiveInputWrapper *input = input_and_card.first;
+			input->connect_card(input_and_card.second, input_state);
 		}
 		for (const auto &input_and_filename : images_to_select) {
 			input_and_filename.first->switch_image(input_and_filename.second);
@@ -625,10 +748,9 @@ Scene::get_chain(Theme *theme, lua_State *L, unsigned num, const InputState &inp
 bool display(Block *block, lua_State *L, int idx)
 {
 	if (lua_isnumber(L, idx)) {
-		Theme *theme = get_theme_updata(L);
 		int signal_idx = luaL_checknumber(L, idx);
 		block->signal_type_to_connect = Block::CONNECT_SIGNAL;
-		block->signal_to_connect = theme->map_signal(signal_idx);
+		block->signal_to_connect = signal_idx;
 		block->currently_chosen_alternative = find_index_of(block, LIVE_INPUT_YCBCR);  // Will be changed to deinterlaced at get_chain() time if needed.
 		return true;
 #ifdef HAVE_CEF
