@@ -29,6 +29,7 @@
 #include <new>
 #include <utility>
 
+#include "audio_mixer.h"
 #include "defs.h"
 #ifdef HAVE_CEF
 #include "cef_capture.h"
@@ -38,6 +39,7 @@
 #include "image_input.h"
 #include "input_state.h"
 #include "lua_utils.h"
+#include "mainwindow.h"
 #include "pbo_frame_allocator.h"
 #include "scene.h"
 
@@ -52,6 +54,9 @@ using namespace movit;
 
 extern Mixer *global_mixer;
 
+constexpr unsigned Theme::MenuEntry::CHECKABLE;
+constexpr unsigned Theme::MenuEntry::CHECKED;
+
 Theme *get_theme_updata(lua_State* L)
 {
 	luaL_checktype(L, lua_upvalueindex(1), LUA_TLIGHTUSERDATA);
@@ -97,6 +102,7 @@ InputStateInfo::InputStateInfo(const InputState &input_state)
 		last_is_connected[signal_num] = userdata->last_is_connected;
 		last_frame_rate_nom[signal_num] = userdata->last_frame_rate_nom;
 		last_frame_rate_den[signal_num] = userdata->last_frame_rate_den;
+		last_pixel_format[signal_num] = userdata->pixel_format;
 		has_last_subtitle[signal_num] = userdata->has_last_subtitle;
 		last_subtitle[signal_num] = userdata->last_subtitle;
 	}
@@ -116,6 +122,7 @@ Effect *instantiate_effect(EffectChain *chain, EffectType effect_type)
 	case IDENTITY_EFFECT:
 		return new IdentityEffect;
 	case WHITE_BALANCE_EFFECT:
+	case AUTO_WHITE_BALANCE_EFFECT:
 		return new WhiteBalanceEffect;
 	case RESAMPLE_EFFECT:
 		return new ResampleEffect;
@@ -621,8 +628,8 @@ int InputStateInfo_get_width(lua_State* L)
 	InputStateInfo *input_state_info = get_input_state_info(L, 1);
 
 	Theme *theme = get_theme_updata(L);
-	int signal_num = theme->map_signal(luaL_checknumber(L, 2));
-	lua_pushnumber(L, input_state_info->last_width[signal_num]);
+	int card_idx = theme->map_signal_to_card(luaL_checknumber(L, 2));
+	lua_pushnumber(L, input_state_info->last_width[card_idx]);
 	return 1;
 }
 
@@ -631,8 +638,8 @@ int InputStateInfo_get_height(lua_State* L)
 	assert(lua_gettop(L) == 2);
 	InputStateInfo *input_state_info = get_input_state_info(L, 1);
 	Theme *theme = get_theme_updata(L);
-	int signal_num = theme->map_signal(luaL_checknumber(L, 2));
-	lua_pushnumber(L, input_state_info->last_height[signal_num]);
+	int card_idx = theme->map_signal_to_card(luaL_checknumber(L, 2));
+	lua_pushnumber(L, input_state_info->last_height[card_idx]);
 	return 1;
 }
 
@@ -641,9 +648,9 @@ int InputStateInfo_get_frame_height(lua_State* L)
 	assert(lua_gettop(L) == 2);
 	InputStateInfo *input_state_info = get_input_state_info(L, 1);
 	Theme *theme = get_theme_updata(L);
-	int signal_num = theme->map_signal(luaL_checknumber(L, 2));
-	unsigned height = input_state_info->last_height[signal_num];
-	if (input_state_info->last_interlaced[signal_num]) {
+	int card_idx = theme->map_signal_to_card(luaL_checknumber(L, 2));
+	unsigned height = input_state_info->last_height[card_idx];
+	if (input_state_info->last_interlaced[card_idx]) {
 		height *= 2;
 	}
 	lua_pushnumber(L, height);
@@ -655,8 +662,8 @@ int InputStateInfo_get_interlaced(lua_State* L)
 	assert(lua_gettop(L) == 2);
 	InputStateInfo *input_state_info = get_input_state_info(L, 1);
 	Theme *theme = get_theme_updata(L);
-	int signal_num = theme->map_signal(luaL_checknumber(L, 2));
-	lua_pushboolean(L, input_state_info->last_interlaced[signal_num]);
+	int card_idx = theme->map_signal_to_card(luaL_checknumber(L, 2));
+	lua_pushboolean(L, input_state_info->last_interlaced[card_idx]);
 	return 1;
 }
 
@@ -665,8 +672,8 @@ int InputStateInfo_get_has_signal(lua_State* L)
 	assert(lua_gettop(L) == 2);
 	InputStateInfo *input_state_info = get_input_state_info(L, 1);
 	Theme *theme = get_theme_updata(L);
-	int signal_num = theme->map_signal(luaL_checknumber(L, 2));
-	lua_pushboolean(L, input_state_info->last_has_signal[signal_num]);
+	int card_idx = theme->map_signal_to_card(luaL_checknumber(L, 2));
+	lua_pushboolean(L, input_state_info->last_has_signal[card_idx]);
 	return 1;
 }
 
@@ -675,8 +682,8 @@ int InputStateInfo_get_is_connected(lua_State* L)
 	assert(lua_gettop(L) == 2);
 	InputStateInfo *input_state_info = get_input_state_info(L, 1);
 	Theme *theme = get_theme_updata(L);
-	int signal_num = theme->map_signal(luaL_checknumber(L, 2));
-	lua_pushboolean(L, input_state_info->last_is_connected[signal_num]);
+	int card_idx = theme->map_signal_to_card(luaL_checknumber(L, 2));
+	lua_pushboolean(L, input_state_info->last_is_connected[card_idx]);
 	return 1;
 }
 
@@ -685,8 +692,8 @@ int InputStateInfo_get_frame_rate_nom(lua_State* L)
 	assert(lua_gettop(L) == 2);
 	InputStateInfo *input_state_info = get_input_state_info(L, 1);
 	Theme *theme = get_theme_updata(L);
-	int signal_num = theme->map_signal(luaL_checknumber(L, 2));
-	lua_pushnumber(L, input_state_info->last_frame_rate_nom[signal_num]);
+	int card_idx = theme->map_signal_to_card(luaL_checknumber(L, 2));
+	lua_pushnumber(L, input_state_info->last_frame_rate_nom[card_idx]);
 	return 1;
 }
 
@@ -695,8 +702,8 @@ int InputStateInfo_get_frame_rate_den(lua_State* L)
 	assert(lua_gettop(L) == 2);
 	InputStateInfo *input_state_info = get_input_state_info(L, 1);
 	Theme *theme = get_theme_updata(L);
-	int signal_num = theme->map_signal(luaL_checknumber(L, 2));
-	lua_pushnumber(L, input_state_info->last_frame_rate_den[signal_num]);
+	int card_idx = theme->map_signal_to_card(luaL_checknumber(L, 2));
+	lua_pushnumber(L, input_state_info->last_frame_rate_den[card_idx]);
 	return 1;
 }
 
@@ -705,11 +712,11 @@ int InputStateInfo_get_last_subtitle(lua_State* L)
 	assert(lua_gettop(L) == 2);
 	InputStateInfo *input_state_info = get_input_state_info(L, 1);
 	Theme *theme = get_theme_updata(L);
-	int signal_num = theme->map_signal(luaL_checknumber(L, 2));
-	if (!input_state_info->has_last_subtitle[signal_num]) {
+	int card_idx = theme->map_signal_to_card(luaL_checknumber(L, 2));
+	if (!input_state_info->has_last_subtitle[card_idx]) {
 		lua_pushnil(L);
 	} else {
-		lua_pushstring(L, input_state_info->last_subtitle[signal_num].c_str());
+		lua_pushstring(L, input_state_info->last_subtitle[card_idx].c_str());
 	}
 	return 1;
 }
@@ -754,22 +761,22 @@ int InputStateInfo_get_human_readable_resolution(lua_State* L)
 	assert(lua_gettop(L) == 2);
 	InputStateInfo *input_state_info = get_input_state_info(L, 1);
 	Theme *theme = get_theme_updata(L);
-	int signal_num = theme->map_signal(luaL_checknumber(L, 2));
+	int card_idx = theme->map_signal_to_card(luaL_checknumber(L, 2));
 
 	string str;
-	if (!input_state_info->last_is_connected[signal_num]) {
+	if (!input_state_info->last_is_connected[card_idx]) {
 		str = "disconnected";
-	} else if (input_state_info->last_height[signal_num] <= 0) {
+	} else if (input_state_info->last_height[card_idx] <= 0) {
 		str = "no signal";
-	} else if (!input_state_info->last_has_signal[signal_num]) {
-		if (input_state_info->last_height[signal_num] == 525) {
+	} else if (!input_state_info->last_has_signal[card_idx]) {
+		if (input_state_info->last_height[card_idx] == 525) {
 			// Special mode for the USB3 cards.
 			str = "no signal";
 		} else {
-			str = get_human_readable_resolution(input_state_info, signal_num) + ", no signal";
+			str = get_human_readable_resolution(input_state_info, card_idx) + ", no signal";
 		}
 	} else {
-		str = get_human_readable_resolution(input_state_info, signal_num);
+		str = get_human_readable_resolution(input_state_info, card_idx);
 	}
 
 	lua_pushstring(L, str.c_str());
@@ -860,6 +867,7 @@ const luaL_Reg Scene_funcs[] = {
 	{ "new", Scene_new },
 	{ "__gc", Scene_gc },
 	{ "add_input", Scene::add_input },
+	{ "add_white_balance", Scene::add_white_balance },
 	{ "add_effect", Scene::add_effect },
 	{ "add_optional_effect", Scene::add_optional_effect },
 	{ "finalize", Scene::finalize },
@@ -1116,14 +1124,14 @@ bool LiveInputWrapper::connect_signal(int signal_num)
 		return true;
 	}
 
-	signal_num = theme->map_signal(signal_num);
-	connect_signal_raw(signal_num, *theme->input_state);
+	int card_idx = theme->map_signal_to_card(signal_num);
+	connect_card(card_idx, *theme->input_state);
 	return true;
 }
 
-void LiveInputWrapper::connect_signal_raw(int signal_num, const InputState &input_state)
+void LiveInputWrapper::connect_card(int card_idx, const InputState &input_state)
 {
-	BufferedFrame first_frame = input_state.buffered_frames[signal_num][0];
+	BufferedFrame first_frame = input_state.buffered_frames[card_idx][0];
 	if (first_frame.frame == nullptr) {
 		// No data yet.
 		return;
@@ -1138,10 +1146,10 @@ void LiveInputWrapper::connect_signal_raw(int signal_num, const InputState &inpu
 		}
 	}
 
-	movit::YCbCrLumaCoefficients ycbcr_coefficients = input_state.ycbcr_coefficients[signal_num];
-	bool full_range = input_state.full_range[signal_num];
+	movit::YCbCrLumaCoefficients ycbcr_coefficients = input_state.ycbcr_coefficients[card_idx];
+	bool full_range = input_state.full_range[card_idx];
 
-	if (input_state.ycbcr_coefficients_auto[signal_num]) {
+	if (input_state.ycbcr_coefficients_auto[card_idx]) {
 		full_range = false;
 
 		// The Blackmagic driver docs claim that the device outputs Y'CbCr
@@ -1165,7 +1173,7 @@ void LiveInputWrapper::connect_signal_raw(int signal_num, const InputState &inpu
 
 	BufferedFrame last_good_frame = first_frame;
 	for (unsigned i = 0; i < max(ycbcr_inputs.size(), rgba_inputs.size()); ++i) {
-		BufferedFrame frame = input_state.buffered_frames[signal_num][i];
+		BufferedFrame frame = input_state.buffered_frames[card_idx][i];
 		if (frame.frame == nullptr) {
 			// Not enough data; reuse last frame (well, field).
 			// This is suboptimal, but we have nothing better.
@@ -1194,7 +1202,14 @@ void LiveInputWrapper::connect_signal_raw(int signal_num, const InputState &inpu
 			ycbcr_inputs[i]->set_texture_num(0, userdata->tex_y[frame.field_number]);
 			ycbcr_inputs[i]->set_texture_num(1, userdata->tex_cb[frame.field_number]);
 			ycbcr_inputs[i]->set_texture_num(2, userdata->tex_cr[frame.field_number]);
-			ycbcr_inputs[i]->change_ycbcr_format(userdata->ycbcr_format);
+			// YCbCrPlanar is used for video streams, where we can have metadata from the mux.
+			// Prefer that if there's no override. (Overrides are only available when using
+			// video as SRT cards.)
+			if (input_state.ycbcr_coefficients_auto[card_idx]) {
+				ycbcr_inputs[i]->change_ycbcr_format(userdata->ycbcr_format);
+			} else {
+				ycbcr_inputs[i]->change_ycbcr_format(input_ycbcr_format);
+			}
 			ycbcr_inputs[i]->set_width(width);
 			ycbcr_inputs[i]->set_height(height);
 			break;
@@ -1217,7 +1232,7 @@ void LiveInputWrapper::connect_signal_raw(int signal_num, const InputState &inpu
 	}
 
 	if (deinterlace) {
-		BufferedFrame frame = input_state.buffered_frames[signal_num][0];
+		BufferedFrame frame = input_state.buffered_frames[card_idx][0];
 		CHECK(deinterlace_effect->set_int("current_field_position", frame.field_number));
 	}
 }
@@ -1293,8 +1308,166 @@ int Nageru_set_supports_wb(lua_State *L)
 	return 0;
 }
 
-Theme::Theme(const string &filename, const vector<string> &search_dirs, ResourcePool *resource_pool, unsigned num_cards)
-	: resource_pool(resource_pool), num_cards(num_cards), signal_to_card_mapping(global_flags.default_stream_mapping)
+// NOTE: There's a race condition in all of the audio functions; if the mapping
+// is changed by the user underway, you might not be manipulating the bus you
+// expect. (You should not get crashes, though.) There's not all that much we
+// can do about it, short of locking the entire mixer while anything from the
+// theme runs.
+
+int Nageru_get_num_audio_buses(lua_State *L)
+{
+	if (global_audio_mixer == nullptr) {
+		// The audio mixer isn't set up until we know how many FFmpeg inputs we have.
+		luaL_error(L, "Audio functions can not be called before the theme is done initializing.");
+	}
+	lua_pushinteger(L, global_audio_mixer->num_buses());
+	return 1;
+}
+
+int Nageru_get_audio_bus_name(lua_State *L)
+{
+	if (global_audio_mixer == nullptr) {
+		// The audio mixer isn't set up until we know how many FFmpeg inputs we have.
+		luaL_error(L, "Audio functions can not be called before the theme is done initializing.");
+	}
+	int bus_index = luaL_checknumber(L, 1);
+	InputMapping input_mapping = global_audio_mixer->get_input_mapping();
+	if (bus_index < 0 || size_t(bus_index) >= input_mapping.buses.size()) {
+		// Doesn't fix the race, but fixes other out-of-bounds.
+		print_warning(L, "Theme called get_audio_bus_name() on nonexistent bus %d; returning nil.\n", bus_index);
+		lua_pushnil(L);
+	} else {
+		lua_pushstring(L, input_mapping.buses[bus_index].name.c_str());
+	}
+	return 1;
+}
+
+int Nageru_get_audio_bus_fader_level_db(lua_State *L)
+{
+	if (global_audio_mixer == nullptr) {
+		// The audio mixer isn't set up until we know how many FFmpeg inputs we have.
+		luaL_error(L, "Audio functions can not be called before the theme is done initializing.");
+	}
+
+	int bus_index = luaL_checknumber(L, 1);
+	if (bus_index < 0 || size_t(bus_index) >= global_audio_mixer->num_buses()) {
+		// Doesn't fix the race, but fixes other out-of-bounds.
+		print_warning(L, "Theme called get_audio_bus_fader_level_db() on nonexistent bus %d; returning 0.0.\n", bus_index);
+		lua_pushnumber(L, 0.0);
+	} else {
+		lua_pushnumber(L, global_audio_mixer->get_fader_volume(bus_index));
+	}
+	return 1;
+}
+
+int Nageru_set_audio_bus_fader_level_db(lua_State *L)
+{
+	if (global_audio_mixer == nullptr || global_mainwindow == nullptr) {
+		// The audio mixer isn't set up until we know how many FFmpeg inputs we have.
+		luaL_error(L, "Audio functions can not be called before the theme is done initializing.");
+	}
+
+	int bus_index = luaL_checknumber(L, 1);
+	if (bus_index < 0 || size_t(bus_index) >= global_audio_mixer->num_buses()) {
+		// Doesn't fix the race, but fixes other out-of-bounds.
+		print_warning(L, "Theme called set_audio_bus_fader_level_db() on nonexistent bus %d; ignoring.\n", bus_index);
+		return 0;
+	}
+	double level_db = luaL_checknumber(L, 2);
+
+	// Go through the UI, so that it gets updated.
+	global_mainwindow->set_fader_absolute(bus_index, level_db);
+	return 0;
+}
+
+int Nageru_get_audio_bus_mute(lua_State *L)
+{
+	if (global_audio_mixer == nullptr) {
+		// The audio mixer isn't set up until we know how many FFmpeg inputs we have.
+		luaL_error(L, "Audio functions can not be called before the theme is done initializing.");
+	}
+
+	int bus_index = luaL_checknumber(L, 1);
+	if (bus_index < 0 || size_t(bus_index) >= global_audio_mixer->num_buses()) {
+		// Doesn't fix the race, but fixes other out-of-bounds.
+		print_warning(L, "Theme called get_audio_bus_mute() on nonexistent bus %d; returning false.\n", bus_index);
+		lua_pushboolean(L, false);
+	} else {
+		lua_pushboolean(L, global_audio_mixer->get_mute(bus_index));
+	}
+	return 1;
+}
+
+int Nageru_set_audio_bus_mute(lua_State *L)
+{
+	if (global_audio_mixer == nullptr || global_mainwindow == nullptr) {
+		// The audio mixer isn't set up until we know how many FFmpeg inputs we have.
+		luaL_error(L, "Audio functions can not be called before the theme is done initializing.");
+	}
+
+	int bus_index = luaL_checknumber(L, 1);
+	if (bus_index < 0 || size_t(bus_index) >= global_audio_mixer->num_buses()) {
+		// Doesn't fix the race, but fixes other out-of-bounds.
+		print_warning(L, "Theme called set_audio_bus_mute() on nonexistent bus %d; ignoring.\n", bus_index);
+		return 0;
+	}
+	bool mute = checkbool(L, 2);
+
+	// Go through the UI, so that it gets updated.
+	if (mute != global_audio_mixer->get_mute(bus_index)) {
+		global_mainwindow->toggle_mute(bus_index);
+	}
+	return 0;
+}
+
+int Nageru_get_audio_bus_eq_level_db(lua_State *L)
+{
+	if (global_audio_mixer == nullptr) {
+		// The audio mixer isn't set up until we know how many FFmpeg inputs we have.
+		luaL_error(L, "Audio functions can not be called before the theme is done initializing.");
+	}
+
+	int bus_index = luaL_checknumber(L, 1);
+	int band = luaL_checknumber(L, 2);
+	if (bus_index < 0 || size_t(bus_index) >= global_audio_mixer->num_buses()) {
+		// Doesn't fix the race, but fixes other out-of-bounds.
+		print_warning(L, "Theme called get_audio_bus_eq_level_db() on nonexistent bus %d; returning 0.0.\n", bus_index);
+		lua_pushnumber(L, 0.0);
+	} else if (band != EQ_BAND_BASS && band != EQ_BAND_MID && band != EQ_BAND_TREBLE) {
+		print_warning(L, "Theme called get_audio_bus_eq_level_db() on nonexistent band; returning 0.0.\n", bus_index);
+		lua_pushnumber(L, 0.0);
+	} else {
+		lua_pushnumber(L, global_audio_mixer->get_eq(bus_index, EQBand(band)));
+	}
+	return 1;
+}
+
+int Nageru_set_audio_bus_eq_level_db(lua_State *L)
+{
+	if (global_audio_mixer == nullptr || global_mainwindow == nullptr) {
+		// The audio mixer isn't set up until we know how many FFmpeg inputs we have.
+		luaL_error(L, "Audio functions can not be called before the theme is done initializing.");
+	}
+
+	int bus_index = luaL_checknumber(L, 1);
+	int band = luaL_checknumber(L, 2);
+	if (bus_index < 0 || size_t(bus_index) >= global_audio_mixer->num_buses()) {
+		// Doesn't fix the race, but fixes other out-of-bounds.
+		print_warning(L, "Theme called set_audio_bus_eq_level_db() on nonexistent bus %d; ignoring.\n", bus_index);
+		return 0;
+	} else if (band != EQ_BAND_BASS && band != EQ_BAND_MID && band != EQ_BAND_TREBLE) {
+		print_warning(L, "Theme called set_audio_bus_eq_level_db() on nonexistent band; returning 0.0.\n", bus_index);
+		return 0;
+	}
+	double level_db = luaL_checknumber(L, 3);
+
+	// Go through the UI, so that it gets updated.
+	global_mainwindow->set_eq_absolute(bus_index, EQBand(band), level_db);
+	return 0;
+}
+
+Theme::Theme(const string &filename, const vector<string> &search_dirs, ResourcePool *resource_pool)
+	: resource_pool(resource_pool), signal_to_card_mapping(global_flags.default_stream_mapping)
 {
 	// Defaults.
 	channel_names[0] = "Live";
@@ -1419,6 +1592,9 @@ void Theme::register_globals()
 		{ "VIDEO_FORMAT_YCBCR", bmusb::PixelFormat_8BitYCbCrPlanar },
 		{ "CHECKABLE", MenuEntry::CHECKABLE },
 		{ "CHECKED", MenuEntry::CHECKED },
+		{ "EQ_BAND_BASS", EQ_BAND_BASS },
+		{ "EQ_BAND_MID", EQ_BAND_MID },
+		{ "EQ_BAND_TREBLE", EQ_BAND_TREBLE },
 	};
 	const vector<pair<string, string>> str_constants = {
 		{ "THEME_PATH", theme_path },
@@ -1438,11 +1614,23 @@ void Theme::register_globals()
 	}
 
 	const luaL_Reg Nageru_funcs[] = {
+		// Channel information.
 		{ "set_channel_name", Nageru_set_channel_name },
 		{ "set_num_channels", Nageru_set_num_channels },
 		{ "set_channel_signal", Nageru_set_channel_signal },
 		{ "set_supports_wb", Nageru_set_supports_wb },
-		{ NULL, NULL }
+
+		// Audio.
+		{ "get_num_audio_buses", Nageru_get_num_audio_buses },
+		{ "get_audio_bus_name", Nageru_get_audio_bus_name },
+		{ "get_audio_bus_fader_level_db", Nageru_get_audio_bus_fader_level_db },
+		{ "set_audio_bus_fader_level_db", Nageru_set_audio_bus_fader_level_db },
+		{ "get_audio_bus_eq_level_db", Nageru_get_audio_bus_eq_level_db },
+		{ "set_audio_bus_eq_level_db", Nageru_set_audio_bus_eq_level_db },
+		{ "get_audio_bus_mute", Nageru_get_audio_bus_mute },
+		{ "set_audio_bus_mute", Nageru_set_audio_bus_mute },
+
+		{ nullptr, nullptr }
 	};
 	lua_pushlightuserdata(L, this);
 	luaL_setfuncs(L, Nageru_funcs, 1);        // for (name,f in funcs) { mt[name] = f, with upvalue {theme} }
@@ -1497,13 +1685,13 @@ Theme::Chain Theme::get_chain_from_effect_chain(EffectChain *effect_chain, unsig
 		// each FFmpeg or CEF input, so we'll do it here.
 		if (video_signal_connections.count(effect_chain)) {
 			for (const VideoSignalConnection &conn : video_signal_connections[effect_chain]) {
-				conn.wrapper->connect_signal_raw(conn.source->get_card_index(), input_state);
+				conn.wrapper->connect_card(conn.source->get_card_index(), input_state);
 			}
 		}
 #ifdef HAVE_CEF
 		if (html_signal_connections.count(effect_chain)) {
 			for (const CEFSignalConnection &conn : html_signal_connections[effect_chain]) {
-				conn.wrapper->connect_signal_raw(conn.source->get_card_index(), input_state);
+				conn.wrapper->connect_card(conn.source->get_card_index(), input_state);
 			}
 		}
 #endif
@@ -1560,8 +1748,8 @@ Theme::Chain Theme::get_chain(unsigned num, float t, unsigned width, unsigned he
 
 	// TODO: Can we do better, e.g. by running setup_chain() and seeing what it references?
 	// Actually, setup_chain does maybe hold all the references we need now anyway?
-	chain.input_frames.reserve(num_cards * FRAME_HISTORY_LENGTH);
-	for (unsigned card_index = 0; card_index < num_cards; ++card_index) {
+	chain.input_frames.reserve(MAX_VIDEO_CARDS * FRAME_HISTORY_LENGTH);
+	for (unsigned card_index = 0; card_index < MAX_VIDEO_CARDS; ++card_index) {
 		for (unsigned frame_num = 0; frame_num < FRAME_HISTORY_LENGTH; ++frame_num) {
 			chain.input_frames.push_back(input_state.buffered_frames[card_index][frame_num].frame);
 		}
@@ -1608,7 +1796,7 @@ string Theme::get_channel_name(unsigned channel)
 	return retstr;
 }
 
-int Theme::get_channel_signal(unsigned channel)
+int Theme::map_channel_to_signal(unsigned channel)
 {
 	lock_guard<mutex> lock(m);
 	lua_getglobal(L, "channel_signal");
@@ -1682,10 +1870,40 @@ bool Theme::get_supports_set_wb(unsigned channel)
 	return ret;
 }
 
-void Theme::set_wb(unsigned channel, double r, double g, double b)
+void Theme::set_wb(unsigned channel, float r, float g, float b)
+{
+	int signal = map_channel_to_signal(channel);
+
+	lock_guard<mutex> lock(m);
+	if (signal != -1) {
+		int card_idx = map_signal_to_card(signal);
+		white_balance_for_card[card_idx] = RGBTriplet{ r, g, b };
+	}
+
+	call_lua_wb_callback(channel, r, g, b);
+}
+
+void Theme::set_wb_for_card(int card_idx, float r, float g, float b)
 {
 	lock_guard<mutex> lock(m);
+	white_balance_for_card[card_idx] = RGBTriplet{ r, g, b };
+
+	for (const auto &channel_and_signal : channel_signals) {
+		if (map_signal_to_card(channel_and_signal.second) == card_idx) {
+			call_lua_wb_callback(channel_and_signal.first, r, g, b);
+		}
+	}
+}
+
+void Theme::call_lua_wb_callback(unsigned channel, float r, float g, float b)
+{
 	lua_getglobal(L, "set_wb");
+	if (lua_isnil(L, -1)) {
+		// The function doesn't exist, to just ignore. We've stored the white balance,
+		// and most likely, it will be picked up by auto white balance instead.
+		lua_pop(L, 1);
+		return;
+	}
 	lua_pushnumber(L, channel);
 	lua_pushnumber(L, r);
 	lua_pushnumber(L, g);
@@ -1698,6 +1916,15 @@ void Theme::set_wb(unsigned channel, double r, double g, double b)
 	assert(lua_gettop(L) == 0);
 }
 
+RGBTriplet Theme::get_white_balance_for_card(int card_idx)
+{
+	if (white_balance_for_card.count(card_idx)) {
+		return white_balance_for_card[card_idx];
+	} else {
+		return RGBTriplet{ 1.0, 1.0, 1.0 };
+	}
+}
+
 vector<string> Theme::get_transition_names(float t)
 {
 	lock_guard<mutex> lock(m);
@@ -1719,7 +1946,7 @@ vector<string> Theme::get_transition_names(float t)
 	return ret;
 }
 
-int Theme::map_signal(int signal_num)
+int Theme::map_signal_to_card(int signal_num)
 {
 	// Negative numbers map to raw signals.
 	if (signal_num < 0) {
@@ -1732,33 +1959,34 @@ int Theme::map_signal(int signal_num)
 	}
 
 	int card_index;
-	if (global_flags.output_card != -1 && num_cards > 1) {
+	if (global_flags.output_card != -1) {
 		// Try to exclude the output card from the default card_index.
-		card_index = signal_num % (num_cards - 1);
+		card_index = signal_num % (global_flags.max_num_cards - 1);
 		if (card_index >= global_flags.output_card) {
 			 ++card_index;
 		}
-		if (signal_num >= int(num_cards - 1)) {
+		if (signal_num >= int(global_flags.max_num_cards - 1)) {
 			print_warning(L, "Theme asked for input %d, but we only have %u input card(s) (card %d is busy with output).\n",
-				signal_num, num_cards - 1, global_flags.output_card);
+				signal_num, global_flags.max_num_cards - 1, global_flags.output_card);
 			fprintf(stderr, "Mapping to card %d instead.\n", card_index);
 		}
 	} else {
-		card_index = signal_num % num_cards;
-		if (signal_num >= int(num_cards)) {
-			print_warning(L, "Theme asked for input %d, but we only have %u card(s).\n", signal_num, num_cards);
+		card_index = signal_num % global_flags.max_num_cards;
+		if (signal_num >= int(global_flags.max_num_cards)) {
+			print_warning(L, "Theme asked for input %d, but we only have %u card(s).\n", signal_num, global_flags.max_num_cards);
 			fprintf(stderr, "Mapping to card %d instead.\n", card_index);
 		}
 	}
+	global_mixer->force_card_active(card_index);
 	signal_to_card_mapping[signal_num] = card_index;
 	return card_index;
 }
 
-void Theme::set_signal_mapping(int signal_num, int card_num)
+void Theme::set_signal_mapping(int signal_num, int card_idx)
 {
 	lock_guard<mutex> lock(map_m);
-	assert(card_num < int(num_cards));
-	signal_to_card_mapping[signal_num] = card_num;
+	assert(card_idx < MAX_VIDEO_CARDS);
+	signal_to_card_mapping[signal_num] = card_idx;
 }
 
 void Theme::transition_clicked(int transition_num, float t)
@@ -1900,3 +2128,15 @@ string Theme::format_status_line(const string &disk_space_left_text, double file
 	assert(lua_gettop(L) == 0);
 	return text;
 }
+
+void Theme::remove_card(unsigned card_index)
+{
+	lock_guard<mutex> lock(map_m);
+	for (auto it = signal_to_card_mapping.begin(); it != signal_to_card_mapping.end(); ) {
+		if (it->second == int(card_index)) {
+			it = signal_to_card_mapping.erase(it);
+		} else {
+			++it;
+		}
+	}
+}
