@@ -59,11 +59,17 @@ VideoEncoder::VideoEncoder(ResourcePool *resource_pool, QSurface *surface, const
 		stream_audio_encoder.reset(new AudioEncoder(global_flags.stream_audio_codec_name, global_flags.stream_audio_codec_bitrate, oformat));
 	}
 	if (global_flags.x264_video_to_http || global_flags.x264_video_to_disk) {
-		x264_encoder.reset(new X264Encoder(oformat));
+		x264_encoder.reset(new X264Encoder(oformat, /*use_separate_disk_params=*/false));
+	}
+	X264Encoder *http_encoder = x264_encoder.get();
+	X264Encoder *disk_encoder = x264_encoder.get();
+	if (global_flags.x264_separate_disk_encode) {
+		x264_disk_encoder.reset(new X264Encoder(oformat, /*use_separate_disk_params=*/true));
+		disk_encoder = x264_disk_encoder.get();
 	}
 
 	string filename = generate_local_dump_filename(/*frame=*/0);
-	quicksync_encoder.reset(new QuickSyncEncoder(filename, resource_pool, surface, va_display, width, height, oformat, x264_encoder.get(), disk_space_estimator));
+	quicksync_encoder.reset(new QuickSyncEncoder(filename, resource_pool, surface, va_display, width, height, oformat, http_encoder, disk_encoder, disk_space_estimator));
 
 	open_output_stream();
 	stream_audio_encoder->add_mux(stream_mux.get());
@@ -77,6 +83,7 @@ VideoEncoder::~VideoEncoder()
 {
 	quicksync_encoder->shutdown();
 	x264_encoder.reset(nullptr);
+	x264_disk_encoder.reset(nullptr);
 	quicksync_encoder->close_file();
 	quicksync_encoder.reset(nullptr);
 	while (quicksync_encoders_in_shutdown.load() > 0) {
@@ -100,12 +107,17 @@ void VideoEncoder::do_cut(int frame)
 	lock_guard<mutex> lock1(qs_mu, adopt_lock), lock2(qs_audio_mu, adopt_lock);
 	QuickSyncEncoder *old_encoder = quicksync_encoder.release();  // When we go C++14, we can use move capture instead.
 	X264Encoder *old_x264_encoder = nullptr;
+	X264Encoder *old_x264_disk_encoder = nullptr;
 	if (global_flags.x264_video_to_disk) {
 		old_x264_encoder = x264_encoder.release();
 	}
-	thread([old_encoder, old_x264_encoder, this]{
+	if (global_flags.x264_separate_disk_encode) {
+		old_x264_disk_encoder = x264_disk_encoder.release();
+	}
+	thread([old_encoder, old_x264_encoder, old_x264_disk_encoder, this]{
 		old_encoder->shutdown();
 		delete old_x264_encoder;
+		delete old_x264_disk_encoder;
 		old_encoder->close_file();
 		stream_mux->unplug();
 
@@ -116,7 +128,8 @@ void VideoEncoder::do_cut(int frame)
 	}).detach();
 
 	if (global_flags.x264_video_to_disk) {
-		x264_encoder.reset(new X264Encoder(oformat));
+		x264_encoder.reset(new X264Encoder(oformat, /*use_separate_disk_params=*/false));
+		assert(global_flags.x264_video_to_http);
 		if (global_flags.x264_video_to_http) {
 			x264_encoder->add_mux(stream_mux.get());
 		}
@@ -124,8 +137,14 @@ void VideoEncoder::do_cut(int frame)
 			x264_encoder->change_bitrate(overriding_bitrate);
 		}
 	}
+	X264Encoder *http_encoder = x264_encoder.get();
+	X264Encoder *disk_encoder = x264_encoder.get();
+	if (global_flags.x264_separate_disk_encode) {
+		x264_disk_encoder.reset(new X264Encoder(oformat, /*use_separate_disk_params=*/true));
+		disk_encoder = x264_disk_encoder.get();
+	}
 
-	quicksync_encoder.reset(new QuickSyncEncoder(filename, resource_pool, surface, va_display, width, height, oformat, x264_encoder.get(), disk_space_estimator));
+	quicksync_encoder.reset(new QuickSyncEncoder(filename, resource_pool, surface, va_display, width, height, oformat, http_encoder, disk_encoder, disk_space_estimator));
 	quicksync_encoder->set_stream_mux(stream_mux.get());
 }
 
@@ -172,7 +191,7 @@ RefCountedGLsync VideoEncoder::end_frame()
 void VideoEncoder::open_output_stream()
 {
 	AVFormatContext *avctx = avformat_alloc_context();
-	avctx->oformat = oformat;
+	avctx->oformat = const_cast<decltype(avctx->oformat)>(oformat);  // const_cast is a hack to work in FFmpeg both before and after 5.0.
 
 	uint8_t *buf = (uint8_t *)av_malloc(MUX_BUFFER_SIZE);
 	avctx->pb = avio_alloc_context(buf, MUX_BUFFER_SIZE, 1, this, nullptr, nullptr, nullptr);
