@@ -78,7 +78,7 @@ private:
 	static int write_packet2_thunk(void *opaque, uint8_t *buf, int buf_size, AVIODataMarkerType type, int64_t time);
 	int write_packet2(uint8_t *buf, int buf_size, AVIODataMarkerType type, int64_t time);
 
-	AVOutputFormat *oformat;
+	const AVOutputFormat *oformat;
 	mutable std::mutex qs_mu, qs_audio_mu;
 	std::unique_ptr<QuickSyncEncoder> quicksync_encoder;  // Under <qs_mu> _and_ <qs_audio_mu>.
 	movit::ResourcePool *resource_pool;
@@ -93,6 +93,7 @@ private:
 	std::unique_ptr<Mux> stream_mux;  // To HTTP.
 	std::unique_ptr<AudioEncoder> stream_audio_encoder;
 	std::unique_ptr<X264Encoder> x264_encoder;  // nullptr if not using x264.
+	std::unique_ptr<X264Encoder> x264_disk_encoder;  // nullptr if not using x264, or if not having separate disk encodes.
 
 	std::string stream_mux_header;
 	MuxMetrics stream_mux_metrics;
