@@ -30,17 +30,17 @@ typedef std::unique_ptr<AVFormatContext, avformat_close_input_unique>
 	AVFormatContextWithCloser;
 
 AVFormatContextWithCloser avformat_open_input_unique(
-	const char *pathname, AVInputFormat *fmt,
+	const char *pathname, const AVInputFormat *fmt,
 	AVDictionary **options);
 
 AVFormatContextWithCloser avformat_open_input_unique(
-	const char *pathname, AVInputFormat *fmt,
+	const char *pathname, const AVInputFormat *fmt,
 	AVDictionary **options,
 	const AVIOInterruptCB &interrupt_cb);
 
 AVFormatContextWithCloser avformat_open_input_unique(
 	int (*read_packet)(void *opaque, uint8_t *buf, int buf_size),
-	void *opaque, AVInputFormat *fmt, AVDictionary **options,
+	void *opaque, const AVInputFormat *fmt, AVDictionary **options,
 	const AVIOInterruptCB &interrupt_cb);
 
 
